#include "Artboard.h"

namespace VariableEditor {
void Artboard::setup(){
    props.width = ofGetWidth();
    props.height = ofGetHeight();

    position = glm::vec3(0);

    fboSettings.width = makeEven(props.width * props.pixelDensity);
    fboSettings.height = makeEven(props.height * props.pixelDensity);
    fboSettings.numSamples = 0;
    fboSettings.internalformat = GL_RGBA;
    fboSettings.minFilter = GL_LINEAR_MIPMAP_LINEAR;
    fboSettings.maxFilter = GL_NEAREST;
    //fboSettings.textureTarget = GL_TEXTURE_2D_MULTISAMPLE;
    fbo.allocate(fboSettings);

    fbo.begin();
    ofClear(ofFloatColor(props.backgroundColor[0],
                         props.backgroundColor[1],
                         props.backgroundColor[2],
                         props.backgroundColor[3]));
    fbo.end();
}
void Artboard::setProps(const ArtboardProps & props){
    if(props.width != this->props.width
       || props.height != this->props.height
       || props.pixelDensity != this->props.pixelDensity){

        ofFbo newFbo;
        fboSettings.width = makeEven(props.width * props.pixelDensity);
        fboSettings.height = makeEven(props.height * props.pixelDensity);
        newFbo.allocate(fboSettings);
        newFbo.resetAnchor();
        newFbo.begin();
        ofClear(ofFloatColor(props.backgroundColor[0],
                             props.backgroundColor[1],
                             props.backgroundColor[2],
                             props.backgroundColor[3]));
        newFbo.end();
        fbo = std::move(newFbo);
    }
    position = glm::vec3(props.x, props.y, 0);
    this->props = props;
}
ArtboardProps Artboard::getProps(){
    return props;
}
void Artboard::setPosition(float x,
                           float y,
                           float z){
    position = glm::vec3(x, y, z);
}
glm::ivec2 Artboard::getShape(){
    return glm::ivec2(props.width, props.height);
}
void Artboard::begin(){
    fbo.begin();
    glEnable(GL_BLEND);
    glBlendEquation(GL_FUNC_ADD);
    glBlendFuncSeparate(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA, GL_ONE, GL_ONE);
    ofPushStyle();
    ofFill();
    ofSetColor(ofFloatColor(props.backgroundColor[0],
                            props.backgroundColor[1],
                            props.backgroundColor[2],
                            props.backgroundColor[3]));
    ofDrawRectangle(0, 0, fbo.getWidth(), fbo.getHeight());
    ofPopStyle();

}
void Artboard::end(){
    fbo.getTexture().generateMipmap();
    fbo.end();
}
void Artboard::draw(){
    ofPushMatrix();
    ofTranslate(position);
    ofScale(props.zoom);
    fbo.draw(0, 0, makeEven(fbo.getWidth() / props.pixelDensity), makeEven(fbo.getHeight() / props.pixelDensity));
    ofPopMatrix();
}
const ofFbo & Artboard::getFbo(){
    return fbo;
}
}
